@@ -6,13 +6,14 @@ import json
 import logging
 from dataclasses import dataclass
 from functools import singledispatch
+from importlib.metadata import version
 from pathlib import Path
 from typing import Any
+from uuid import UUID
 
+import bs4
 import sphinxnotes.strike
-from atsphinx.audioplayer.nodes import (  # pyright: ignore[reportMissingTypeStubs]
-    audio as audio_node,
-)
+from atsphinx.audioplayer.nodes import audio as audio_node
 from beartype import beartype
 from docutils import nodes
 from docutils.nodes import NodeVisitor
@@ -21,12 +22,16 @@ from sphinx.builders.text import TextBuilder
 from sphinx.util import docutils as sphinx_docutils
 from sphinx.util import logging as sphinx_logging
 from sphinx.util.typing import ExtensionMetadata
+from sphinx_iframes import iframe_node
 from sphinx_immaterial.task_lists import checkbox_label
 from sphinx_simplepdf.directives.pdfinclude import (  # pyright: ignore[reportMissingTypeStubs]
     PdfIncludeDirective,
 )
 from sphinx_toolbox.collapse import CollapseNode
+
+# See https://github.com/sphinx-contrib/video/pull/60.
 from sphinxcontrib.video import (  # pyright: ignore[reportMissingTypeStubs]
+    Video,
     video_node,
 )
 from sphinxnotes.strike import strike_node
@@ -37,6 +42,8 @@ from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
+from ultimate_notion.blocks import Divider as UnoDivider
+from ultimate_notion.blocks import Embed as UnoEmbed
 from ultimate_notion.blocks import Equation as UnoEquation
 from ultimate_notion.blocks import Heading as UnoHeading
 from ultimate_notion.blocks import (
@@ -49,6 +56,7 @@ from ultimate_notion.blocks import (
     Heading3 as UnoHeading3,
 )
 from ultimate_notion.blocks import Image as UnoImage
+from ultimate_notion.blocks import LinkToPage as UnoLinkToPage
 from ultimate_notion.blocks import NumberedItem as UnoNumberedItem
 from ultimate_notion.blocks import (
     Paragraph as UnoParagraph,
@@ -66,7 +74,9 @@ from ultimate_notion.blocks import (
 )
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
+from ultimate_notion.obj_api.blocks import LinkToPage as ObjLinkToPage
 from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
+from ultimate_notion.obj_api.objects import PageRef
 from ultimate_notion.rich_text import Text, math, text
 
 _LOGGER = sphinx_logging.getLogger(name=__name__)
@@ -193,6 +203,40 @@ class _NotionPdfIncludeDirective(PdfIncludeDirective):
         return [node]
 
 
+@beartype
+class _LinkToPageNode(nodes.Element):
+    """
+    Custom node for Notion link-to-page blocks.
+    """
+
+
+@beartype
+class _NotionLinkToPageDirective(sphinx_docutils.SphinxDirective):
+    """
+    Link-to-page directive that creates Notion link-to-page blocks.
+    """
+
+    required_arguments = 1
+
+    def run(self) -> list[nodes.Element]:
+        """
+        Create a Notion link-to-page block.
+        """
+        (page_id,) = self.arguments
+
+        if isinstance(self.env.app.builder, NotionBuilder):
+            node = _LinkToPageNode()
+            node.attributes["page_id"] = page_id
+            return [node]
+
+        notion_url = f"https://www.notion.so/{page_id}"
+        reference = nodes.reference(refuri=notion_url, text=notion_url)
+        reference += nodes.Text(data=notion_url)
+        paragraph = nodes.paragraph()
+        paragraph += reference
+        return [paragraph]
+
+
 @dataclass
 class _TableStructure:
     """
@@ -206,15 +250,15 @@ class _TableStructure:
 
 @singledispatch
 @beartype
-def _process_rich_text_node(child: nodes.Node) -> Text:
+def _process_rich_text_node(node: nodes.Node) -> Text:
     """Create Notion rich text from a single ``docutils`` node.
 
     This is the base function for ``singledispatch``. Specific node types
     are handled by registered functions.
     """
     unsupported_child_type_msg = (
-        f"Unsupported node type within text: {type(child).__name__} on line "
-        f"{child.parent.line} in {child.parent.source}."
+        f"Unsupported node type within text: {type(node).__name__} on line "
+        f"{node.parent.line} in {node.parent.source}."
     )
     # We use ``TRY004`` here because we want to raise a
     # ``ValueError`` if the child type is unsupported, not a
@@ -224,12 +268,21 @@ def _process_rich_text_node(child: nodes.Node) -> Text:
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.reference) -> Text:
+def _(node: nodes.line) -> Text:
+    """
+    Process line nodes by creating rich text.
+    """
+    return _create_styled_text_from_node(node=node) + "\n"
+
+
+@beartype
+@_process_rich_text_node.register
+def _(node: nodes.reference) -> Text:
     """
     Process reference nodes by creating linked text.
     """
-    link_url = child.attributes["refuri"]
-    link_text = child.attributes.get("name", link_url)
+    link_url = node.attributes["refuri"]
+    link_text = node.attributes.get("name", link_url)
 
     return text(
         text=link_text,
@@ -242,120 +295,121 @@ def _(child: nodes.reference) -> Text:
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.target) -> Text:
+def _(node: nodes.target) -> Text:
     """
     Process target nodes by returning empty text (targets are skipped).
     """
-    del child  # Target nodes are skipped
+    del node  # Target nodes are skipped
     return Text.from_plain_text(text="")
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.title_reference) -> Text:
+def _(node: nodes.title_reference) -> Text:
     """Process title reference nodes by creating italic text.
 
     We match the behavior of the HTML builder here.
     If you render ``A `B``` in HTML, it will render as ``A <i>B</i>``.
     """
-    return text(text=child.astext(), italic=True)
+    return text(text=node.astext(), italic=True)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.Text) -> Text:
+def _(node: nodes.Text) -> Text:
     """
     Process Text nodes by creating plain text.
     """
-    return text(text=child.astext())
+    return text(text=node.astext())
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.inline) -> Text:
+def _(node: nodes.inline) -> Text:
     """
     Process inline nodes by creating styled text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.strong) -> Text:
+def _(node: nodes.strong) -> Text:
     """
     Process strong nodes by creating bold text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.emphasis) -> Text:
+def _(node: nodes.emphasis) -> Text:
     """
     Process emphasis nodes by creating italic text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.literal) -> Text:
+def _(node: nodes.literal) -> Text:
     """
     Process literal nodes by creating code text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: strike_node) -> Text:
+def _(node: strike_node) -> Text:
     """
     Process strike nodes by creating strikethrough text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.paragraph) -> Text:
+def _(node: nodes.paragraph) -> Text:
     """
     Process paragraph nodes by creating styled text.
     """
-    return _create_styled_text_from_node(child=child)
+    return _create_styled_text_from_node(node=node)
 
 
 @beartype
 @_process_rich_text_node.register
-def _(child: nodes.math) -> Text:
+def _(node: nodes.math) -> Text:
     """
     Process math nodes by creating math rich text.
     """
-    return math(expression=child.astext())
+    return math(expression=node.astext())
 
 
 @beartype
-def _create_styled_text_from_node(*, child: nodes.Element) -> Text:
+def _create_styled_text_from_node(*, node: nodes.Element) -> Text:
     """Create styled text from a node with CSS class support.
 
     This helper function handles the complex styling logic that was
     previously inline in the main function.
     """
-    classes = child.attributes.get("classes", [])
+    classes = node.attributes.get("classes", [])
     bg_color = _background_color_from_css_classes(classes=classes)
     text_color = _color_from_css_classes(classes=classes)
 
     color_mapping = _get_text_color_mapping()
     bg_color_classes = _get_background_color_classes()
 
-    is_bold = isinstance(child, nodes.strong) or "text-bold" in classes
-    is_italic = isinstance(child, nodes.emphasis) or "text-italic" in classes
+    is_bold = isinstance(node, nodes.strong) or "text-bold" in classes
+    is_italic = isinstance(node, nodes.emphasis) or "text-italic" in classes
     is_code = (
-        isinstance(child, nodes.literal)
+        isinstance(node, nodes.literal)
         or "text-mono" in classes
         or "kbd" in classes
+        or "file" in classes
     )
     is_strikethrough = (
-        isinstance(child, strike_node) or "text-strike" in classes
+        isinstance(node, strike_node) or "text-strike" in classes
     )
     is_underline = "text-underline" in classes
 
@@ -366,6 +420,7 @@ def _create_styled_text_from_node(*, child: nodes.Element) -> Text:
         "text-strike",
         "text-underline",
         "kbd",
+        "file",
         *color_mapping.keys(),
         *bg_color_classes,
     }
@@ -379,14 +434,14 @@ def _create_styled_text_from_node(*, child: nodes.Element) -> Text:
         unsupported_style_msg = (
             "Unsupported text style classes: "
             f"{', '.join(unsupported_styles)}. "
-            f"Text on line {child.parent.line} in {child.parent.source} will "
+            f"Text on line {node.parent.line} in {node.parent.source} will "
             "be rendered without styling."
         )
         _LOGGER.warning(unsupported_style_msg)
 
     color: BGColor | Color | None = bg_color or text_color
     return text(
-        text=child.astext(),
+        text=node.astext(),
         bold=is_bold,
         italic=is_italic,
         code=is_code,
@@ -625,11 +680,16 @@ def _process_node_to_blocks(
     Required function for ``singledispatch``.
     """
     del section_level
-    unsupported_node_type_msg = (
-        f"Unsupported node type: {node.tagname} on line "
-        f"{node.line or node.parent.line} in "
-        f"{node.source or node.parent.source}."
-    )
+    line_number = node.line or node.parent.line
+    source = node.source or node.parent.source
+
+    if line_number is not None and source is not None:
+        unsupported_node_type_msg = (
+            f"Unsupported node type: {node.tagname} on line "
+            f"{line_number} in {source}."
+        )
+    else:
+        unsupported_node_type_msg = f"Unsupported node type: {node.tagname}."
     raise NotImplementedError(unsupported_node_type_msg)
 
 
@@ -747,9 +807,14 @@ def _(
     """
     Process block quote nodes by creating Notion Quote blocks.
     """
-    del section_level
-    rich_text = _create_rich_text_from_children(node=node)
-    return [UnoQuote(text=rich_text)]
+    first_child = node.children[0]
+    rich_text = _process_rich_text_node(first_child)
+    quote = UnoQuote(text=rich_text)
+    for child in node.children[1:]:
+        blocks = _process_node_to_blocks(child, section_level=section_level)
+        quote.append(blocks=blocks)
+
+    return [quote]
 
 
 @beartype
@@ -798,7 +863,10 @@ def _(
                 bulleted_item_block.append(blocks=child_blocks)
             result.append(bulleted_item_block)
         else:
-            assert isinstance(first_child, checkbox_label)
+            assert isinstance(first_child, checkbox_label), (
+                first_child.line,
+                first_child.source,
+            )
             label_text_node = list_item.children[1]
             # Get the checked state from the checkbox_label node
             checked = first_child.attributes.get("checked", False)
@@ -846,7 +914,10 @@ def _(
                 block.append(blocks=child_blocks)
             result.append(block)
         else:
-            assert isinstance(first_child, checkbox_label)
+            assert isinstance(first_child, checkbox_label), (
+                first_child.line,
+                first_child.source,
+            )
             label_text_node = list_item.children[1]
             # Get the checked state from the checkbox_label node
             checked = first_child.attributes.get("checked", False)
@@ -1303,6 +1374,26 @@ def _(
     return [UnoPDF(file=ExternalFile(url=pdf_url))]
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: _LinkToPageNode,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """Process link-to-page nodes by creating Notion link-to-page blocks.
+
+    This handles nodes created by our custom NotionLinkToPageDirective.
+    """
+    del section_level
+
+    page_id = node.attributes["page_id"]
+    page_ref = PageRef(page_id=UUID(hex=page_id))
+    obj_link_to_page = ObjLinkToPage(link_to_page=page_ref)
+
+    return [UnoLinkToPage.wrap_obj_ref(obj_link_to_page)]
+
+
 @beartype
 @_process_node_to_blocks.register
 def _(
@@ -1354,6 +1445,30 @@ def _(
     return blocks
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: iframe_node,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process raw nodes, specifically those containing HTML from the extension
+    ``sphinx-iframes``.
+    """
+    del section_level
+
+    # Check if this is an ``iframe`` from ``sphinx-iframes``.
+    # See https://github.com/TeachBooks/sphinx-iframes/issues/9
+    # for making this more robust.
+    soup = bs4.BeautifulSoup(markup=node.rawsource, features="html.parser")
+    iframes = soup.find_all(name="iframe")
+    (iframe,) = iframes
+    url = iframe.get(key="src")
+    assert url is not None
+    return [UnoEmbed(url=str(object=url))]
+
+
 @beartype
 def _process_rest_example_container(
     *,
@@ -1447,6 +1562,38 @@ def _(
     return []
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.line_block,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process line block nodes by creating separate paragraph blocks for each
+    line.
+    """
+    del section_level
+
+    line_text = _create_rich_text_from_children(node=node)
+    return [UnoParagraph(text=line_text)]
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.transition,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process transition nodes by creating Notion Divider blocks.
+    """
+    del node
+    del section_level
+    return [UnoDivider()]
+
+
 @beartype
 class NotionTranslator(NodeVisitor):
     """
@@ -1527,25 +1674,46 @@ def _notion_register_pdf_include_directive(
         )
 
 
+@beartype
+def _notion_register_link_to_page_directive(
+    app: Sphinx,
+) -> None:
+    """
+    Register the link-to-page directive.
+    """
+    del app
+    sphinx_docutils.register_directive(
+        name="notion-link-to-page",
+        directive=_NotionLinkToPageDirective,
+    )
+
+
 @beartype
 def _filter_ulem(record: logging.LogRecord) -> bool:
     """Filter out the warning about the `ulem package already being included`.
 
-    This warning is emitted by ``sphinxcontrib-text-styles`` or
-    ``sphinxnotes.strike`` when the ``ulem`` package is already included.
+    This warning is emitted by ``sphinxnotes.strike`` when the ``ulem`` package
+    is already included (e.g. by ``sphinxcontrib-text-styles``).
 
     Our users may use both of these extensions, so we filter out the
     warning.
 
     See:
 
-    * https://github.com/sphinx-notes/strike/pull/10
-    * https://github.com/martinpriestley/sphinxcontrib-text-styles/pull/1
+    * https://github.com/sphinx-notes/strike/pull/10 (merged, not released)
     """
     msg = record.getMessage()
     return msg != "latex package 'ulem' already included"
 
 
+@beartype
+def _make_static_dir(app: Sphinx) -> None:
+    """
+    We make the ``_static`` directory that ``sphinx-iframes`` expects.
+    """
+    (app.outdir / "_static").mkdir(parents=True, exist_ok=True)
+
+
 @beartype
 def setup(app: Sphinx) -> ExtensionMetadata:
     """
@@ -1559,8 +1727,26 @@ def setup(app: Sphinx) -> ExtensionMetadata:
         callback=_notion_register_pdf_include_directive,
     )
 
+    app.connect(
+        event="builder-inited",
+        callback=_notion_register_link_to_page_directive,
+    )
+
+    app.connect(event="builder-inited", callback=_make_static_dir)
+
     logger = logging.getLogger(name="sphinx.sphinx.registry")
     logger.addFilter(filter=_filter_ulem)
 
     sphinxnotes.strike.SUPPORTED_BUILDERS.append(NotionBuilder)
-    return {"parallel_read_safe": True}
+
+    # that we use. The ``sphinx-iframes`` extension implements a ``video``
+    # directive that we don't use.
+    # Make sure that if they are both enabled, we use the
+    # ``sphinxcontrib.video`` extension.
+    if "sphinxcontrib.video" in app.extensions:
+        app.add_directive(name="video", cls=Video, override=True)
+
+    return {
+        "parallel_read_safe": True,
+        "version": version(distribution_name="sphinx-notionbuilder"),
+    }
