@@ -9,6 +9,7 @@ import textwrap
 from collections.abc import Callable, Collection
 from pathlib import Path
 from typing import Any
+from uuid import UUID
 
 import anstrip
 import pytest
@@ -21,6 +22,8 @@ from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
+from ultimate_notion.blocks import Divider as UnoDivider
+from ultimate_notion.blocks import Embed as UnoEmbed
 from ultimate_notion.blocks import Equation as UnoEquation
 from ultimate_notion.blocks import (
     Heading1 as UnoHeading1,
@@ -32,6 +35,7 @@ from ultimate_notion.blocks import (
     Heading3 as UnoHeading3,
 )
 from ultimate_notion.blocks import Image as UnoImage
+from ultimate_notion.blocks import LinkToPage as UnoLinkToPage
 from ultimate_notion.blocks import NumberedItem as UnoNumberedItem
 from ultimate_notion.blocks import (
     Paragraph as UnoParagraph,
@@ -49,7 +53,9 @@ from ultimate_notion.blocks import (
 )
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
+from ultimate_notion.obj_api.blocks import LinkToPage as ObjLinkToPage
 from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
+from ultimate_notion.obj_api.objects import PageRef
 from ultimate_notion.rich_text import Text, math, text
 
 
@@ -79,11 +85,13 @@ def _assert_rst_converts_to_notion_objects(
     extensions: tuple[str, ...] = ("sphinx_notion",),
     conf_py_content: str = "",
     expected_warnings: Collection[str] = (),
+    confoverrides: dict[str, Any] | None = None,
 ) -> SphinxTestApp:
     """
     ReStructuredText content converts to expected Notion objects via Sphinx
     build process.
     """
+    confoverrides = confoverrides or {}
     srcdir = tmp_path / "src"
     srcdir.mkdir(exist_ok=True)
 
@@ -96,7 +104,7 @@ def _assert_rst_converts_to_notion_objects(
         srcdir=srcdir,
         builddir=tmp_path / "build",
         buildername="notion",
-        confoverrides={"extensions": list(extensions)},
+        confoverrides={"extensions": list(extensions)} | confoverrides,
     )
     app.build()
     assert app.statuscode == 0
@@ -147,6 +155,97 @@ def test_single_paragraph(
     )
 
 
+def test_notion_link_to_page(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    ``notion-link-to-page`` directives become Notion link-to-page blocks.
+    """
+    test_page_id = "12345678-1234-1234-1234-123456789abc"
+
+    rst_content = f"""
+        .. notion-link-to-page:: {test_page_id}
+    """
+
+    page_ref = PageRef(page_id=UUID(hex=test_page_id))
+    obj_link_to_page = ObjLinkToPage(link_to_page=page_ref)
+    expected_objects: list[Block] = [
+        UnoLinkToPage.wrap_obj_ref(obj_link_to_page),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_notion_link_to_page_with_content_around(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    ``notion-link-to-page`` directive works with surrounding content.
+    """
+    test_page_id = "87654321-4321-4321-4321-cba987654321"
+
+    rst_content = f"""
+        This is a paragraph before.
+
+        .. notion-link-to-page:: {test_page_id}
+
+        This is a paragraph after.
+    """
+
+    page_ref = PageRef(page_id=UUID(hex=test_page_id))
+    obj_link_to_page = ObjLinkToPage(link_to_page=page_ref)
+    expected_objects: list[Block] = [
+        UnoParagraph(text=text(text="This is a paragraph before.")),
+        UnoLinkToPage.wrap_obj_ref(obj_link_to_page),
+        UnoParagraph(text=text(text="This is a paragraph after.")),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_notion_link_to_page_html_output(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    ``notion-link-to-page`` directive with HTML builder creates a link.
+    """
+    test_page_id = "12345678-1234-1234-1234-123456789abc"
+    rst_content = f"""
+        .. notion-link-to-page:: {test_page_id}
+    """
+    srcdir = tmp_path / "src"
+    srcdir.mkdir()
+    (srcdir / "conf.py").touch()
+    (srcdir / "index.rst").write_text(data=rst_content)
+    app = make_app(
+        srcdir=srcdir,
+        builddir=tmp_path / "build",
+        buildername="html",
+        confoverrides={"extensions": ["sphinx_notion"]},
+    )
+    app.build()
+    assert app.statuscode == 0
+    index_html = (tmp_path / "build" / "html" / "index.html").read_text()
+    expected_url = f"https://www.notion.so/{test_page_id}"
+    assert expected_url in index_html
+
+
 def test_multiple_paragraphs(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -553,6 +652,56 @@ def test_multiline_quote(
     )
 
 
+def test_multi_paragraph_quote(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Block quotes with multiple paragraphs create Quote blocks with nested
+    paragraph children.
+    """
+    rst_content = """
+        Some content.
+
+            This is the first paragraph
+            with multiple lines
+            in the quote.
+
+            This is a second paragraph
+            with **bold text** and multiple
+            lines as well.
+    """
+    quote = UnoQuote(
+        text=text(
+            text="This is the first paragraph\nwith multiple lines\n"
+            "in the quote."
+        )
+    )
+
+    nested_paragraph = UnoParagraph(
+        text=(
+            text(text="This is a second paragraph\nwith ")
+            + text(text="bold text", bold=True)
+            + text(text=" and multiple\nlines as well.")
+        )
+    )
+
+    quote.append(blocks=[nested_paragraph])
+
+    expected_objects: list[Block] = [
+        UnoParagraph(text=text(text="Some content.")),
+        quote,
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
 def test_table_of_contents(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -1595,7 +1744,6 @@ def test_local_image_file(
         expected_objects=expected_objects,
         make_app=make_app,
         tmp_path=tmp_path,
-        extensions=("sphinx_notion", "sphinxcontrib.video"),
     )
 
 
@@ -1622,7 +1770,7 @@ def test_simple_video(
         expected_objects=expected_objects,
         make_app=make_app,
         tmp_path=tmp_path,
-        extensions=("sphinx_notion", "sphinxcontrib.video"),
+        extensions=("sphinxcontrib.video", "sphinx_notion"),
     )
 
 
@@ -1652,7 +1800,7 @@ def test_video_with_caption(
         expected_objects=expected_objects,
         make_app=make_app,
         tmp_path=tmp_path,
-        extensions=("sphinx_notion", "sphinxcontrib.video"),
+        extensions=("sphinxcontrib.video", "sphinx_notion"),
     )
 
 
@@ -2795,6 +2943,37 @@ def test_kbd_role(
     )
 
 
+def test_file_role(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """The ``:file:`` role creates file path formatting.
+
+    File paths should be rendered as inline code.
+    """
+    rst_content = """
+        Edit the :file:`config.py` file.
+    """
+
+    normal_text1 = text(text="Edit the ")
+    file_text = text(text="config.py", code=True)
+    normal_text2 = text(text=" file.")
+
+    combined_text = normal_text1 + file_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
 def test_unsupported_node_types_in_rich_text(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -2833,39 +3012,39 @@ def setup(app):
         )
 
 
+@pytest.mark.parametrize(
+    argnames=("rst_content", "node_name", "line_number_available"),
+    argvalues=[
+        (".. raw:: html\n\n   <hr width=50 size=10>", "raw", True),
+        (
+            ".. sidebar:: title\n\n   content",
+            "sidebar",
+            False,
+        ),
+    ],
+)
 def test_unsupported_node_types_in_process_node_to_blocks(
     *,
     make_app: Callable[..., SphinxTestApp],
     tmp_path: Path,
+    rst_content: str,
+    node_name: str,
+    line_number_available: bool,
 ) -> None:
     """
     Unsupported node types in _process_node_to_blocks raise
-    NotImplementedError.
+    ``NotImplementedError``.
     """
-    rst_content = """
-        .. unsupported-node::
-    """
-
-    conf_py_content = """
-from docutils import nodes
-from docutils.parsers.rst import Directive
-
-class UnsupportedNode(Directive):
-    def run(self):
-        # Create a custom node type that's not registered
-        node = nodes.Element()
-        node.tagname = 'unsupported_node'
-        return [node]
-
-def setup(app):
-    app.add_directive('unsupported-node', UnsupportedNode)
-    """
-
-    expected_message = (
-        r"^Unsupported node type: unsupported_node on line "
-        rf"1 in "
-        rf"{re.escape(pattern=str(object=tmp_path / 'src' / 'index.rst'))}.$"
-    )
+    index_rst = tmp_path / "src" / "index.rst"
+    # Some nodes do not have a line number available.
+    if line_number_available:
+        expected_message = (
+            rf"^Unsupported node type: {node_name} on line "
+            rf"1 in "
+            rf"{re.escape(pattern=str(object=index_rst))}.$"
+        )
+    else:
+        expected_message = rf"^Unsupported node type: {node_name}.$"
     with pytest.raises(
         expected_exception=NotImplementedError,
         match=expected_message,
@@ -2875,7 +3054,6 @@ def setup(app):
             expected_objects=[],
             make_app=make_app,
             tmp_path=tmp_path,
-            conf_py_content=conf_py_content,
         )
 
 
@@ -3012,3 +3190,128 @@ def test_rest_example_block(
         tmp_path=tmp_path,
         extensions=("sphinx_notion", "sphinx_toolbox.rest_example"),
     )
+
+
+def test_embed_block(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Blocks using the ``iframe`` directive become Notion Embed blocks.
+    """
+    rst_content = """
+        .. iframe:: https://example.com/embed
+           :width: 600
+           :height: 400
+    """
+
+    expected_objects: list[Block] = [UnoEmbed(url="https://example.com/embed")]
+
+    # Create the _static directory that ``sphinx-iframes`` expects
+    static_dir = tmp_path / "build" / "notion" / "_static"
+    static_dir.mkdir(parents=True, exist_ok=True)
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_iframes"),
+    )
+
+
+def test_embed_and_video(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """``sphinx-iframes`` and ``sphinxcontrib.video`` can be used together in
+    this with ``sphinx-notionbuilder``.
+
+    We check this because there was a conflict between the two
+    extensions. See
+    https://github.com/TeachBooks/sphinx-iframes/issues/8.
+    """
+    rst_content = """
+        .. iframe:: https://example.com/embed
+
+        .. video:: https://example.com/video.mp4
+    """
+
+    expected_objects: list[Block] = [
+        UnoEmbed(url="https://example.com/embed"),
+        UnoVideo(file=ExternalFile(url="https://example.com/video.mp4")),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinxcontrib.video", "sphinx_iframes", "sphinx_notion"),
+    )
+
+
+def test_line_block(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Line blocks (created with pipe character) become empty Notion paragraph
+    blocks.
+    """
+    rst_content = """
+        | This is a line block
+        | with multiple lines
+        | preserved exactly as written
+    """
+
+    expected_objects: list[Block] = [
+        UnoParagraph(
+            text=text(text="This is a line block")
+            + text(text="\n")
+            + text(text="with multiple lines")
+            + text(text="\n")
+            + text(text="preserved exactly as written")
+            + text(text="\n")
+        ),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_transition_divider(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Transitions (horizontal rules) become Notion Divider blocks.
+    """
+    rst_content = """
+        First paragraph.
+
+        ----
+
+        Second paragraph.
+    """
+
+    expected_objects: list[Block] = [
+        UnoParagraph(text=text(text="First paragraph.")),
+        UnoDivider(),
+        UnoParagraph(text=text(text="Second paragraph.")),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
